
/*

  This file contains data structure initialization and class methods
  for the GL_palette object.

  This code is lifted directly out of "OpenGL I: Quick Start", by
  Dale Rogerson, Microsoft Developer Network Library.

*/

#include "stdafx.h"
#include "makepal.h"

//
// GL_palette static initializations
//

unsigned char GL_palette::m_threeto8[8] = {
    0, 0111>>1, 0222>>1, 0333>>1, 0444>>1, 0555>>1, 0666>>1, 0377
};
unsigned char GL_palette::m_twoto8[4] = {
   0, 0x55, 0xaa, 0xff
};
unsigned char GL_palette::m_oneto8[2] = {
    0, 255
};

int GL_palette::m_defaultOverride[13] = {
    0, 3, 24, 27, 64, 67, 88, 173, 181, 236, 247, 164, 91
};

PALETTEENTRY GL_palette::m_defaultPalEntry[20] = {
    { 0,   0,   0,    0 }, //0
    { 0x80,0,   0,    0 }, 
    { 0,   0x80,0,    0 }, 
    { 0x80,0x80,0,    0 }, 
    { 0,   0,   0x80, 0 },
    { 0x80,0,   0x80, 0 },
    { 0,   0x80,0x80, 0 },
    { 0xC0,0xC0,0xC0, 0 }, //7

    { 192, 220, 192,  0 }, //8
    { 166, 202, 240,  0 },
    { 255, 251, 240,  0 },
    { 160, 160, 164,  0 }, //11

    { 0x80,0x80,0x80, 0 }, //12
    { 0xFF,0,   0,    0 },
    { 0,   0xFF,0,    0 },
    { 0xFF,0xFF,0,    0 },
    { 0,   0,   0xFF, 0 },
    { 0xFF,0,   0xFF, 0 },
    { 0,   0xFF,0xFF, 0 },
    { 0xFF,0xFF,0xFF, 0 }  //19
  };

//
// ComponentFromIndex
//
unsigned char GL_palette::ComponentFromIndex(int i, UINT nbits, UINT shift)
{
  unsigned char val;
  
  val = (unsigned char) (i >> shift);
  switch (nbits) {
    
  case 1:
    val &= 0x1;
    return m_oneto8[val];
    
  case 2:
    val &= 0x3;
    return m_twoto8[val];
    
  case 3:
    val &= 0x7;
    return m_threeto8[val];
    
  default:
    return 0;
  }
}  // ComponentFromIndex



/*
   CreateRGBPalette

   Magic RGB Palatte creation function from OpenGL I: Quick Start by
   Dale Rogerson.

 */
BOOL GL_palette::CreateRGBPalette(HDC hDC)
{
  //
  // Check to see if we need a palette
  //
  PIXELFORMATDESCRIPTOR pfd;
  int n = GetPixelFormat(hDC);
  
  DescribePixelFormat(hDC, n, sizeof(PIXELFORMATDESCRIPTOR), &pfd);
  
  if (!(pfd.dwFlags & PFD_NEED_PALETTE)) return FALSE ;
  
  TRACE0("Creating palette\r\n") ;
  
  int palette_size = sizeof(LOGPALETTE) + 256 * sizeof(PALETTEENTRY);
    // allocate a log pal and fill it with the color table info
  LOGPALETTE* pPal = (LOGPALETTE*) malloc( palette_size );
  if (!pPal) 
    {
      TRACE("Out of memory for logpal");
      return FALSE;
    }
  
  pPal->palVersion = 0x300; // Windows 3.0
  pPal->palNumEntries = 256; // table size
  
  //
  // Create RGB Palette
  //
  ASSERT( pfd.cColorBits == 8);
  
  n = 1 << pfd.cColorBits;
  for (int i = 0; i < n; i++)
    {
      pPal->palPalEntry[i].peRed =
	ComponentFromIndex(i, pfd.cRedBits, pfd.cRedShift);
      pPal->palPalEntry[i].peGreen =
	ComponentFromIndex(i, pfd.cGreenBits, pfd.cGreenShift);
      pPal->palPalEntry[i].peBlue =
	ComponentFromIndex(i, pfd.cBlueBits, pfd.cBlueShift);
      pPal->palPalEntry[i].peFlags = 0;
    }
  
  //
  // Fix up color table with system colors.
  //
  if ((pfd.cColorBits == 8)                           &&
      (pfd.cRedBits   == 3) && (pfd.cRedShift   == 0) &&
      (pfd.cGreenBits == 3) && (pfd.cGreenShift == 3) &&
      (pfd.cBlueBits  == 2) && (pfd.cBlueShift  == 6)
      )
    {
      for (int j = 1 ; j <= 12 ; j++)
	pPal->palPalEntry[m_defaultOverride[j]] = m_defaultPalEntry[j];
    }
  
  // Delete any existing GDI palette
  if (m_pPal) 
    delete m_pPal ;
  m_pPal = new CPalette ;
  
  BOOL bResult = m_pPal->CreatePalette(pPal);
  free (pPal);
  
  return bResult;
}  // CreateRGBPalette


