
/*===============================<o>=====================================

  This software was written by Ian L. Kaplan, Chief Fat Bear, Bear Products
  International.  Use of this software, for any purpose, is granted on two
  conditions: 

   1. This copyright notice must be included with the software
      or any software derived from it.

   2. The risk of using this software is accepted by the user.  No
      warranty as to its usefulness or functionality is provided
      or implied.  The author and Bear Products International provides
      no support.

   July 1996

 ===============================<o>=====================================*/

//
// sier2View.cpp : implementation of the CSier2View class
//

/* 
   Most of the Windows set-up (e.g., palette initialization, etc...)
   in done in this file.  This file also contains the OnDraw class
   function which redraws the Sierpinski Pyramid.

   The code in this file is based on the code in "OpenGL I: Quick Start"
   by Dale Rogerson, Microsoft Developer Network Library.  In several
   cases below, I have quoted from or paraphrased pieces of this article
   in the comments.

 */

#include "stdafx.h"
#include "makepal.h"
#include "assert.h"

#include "sier2.h"
#include "sier2Doc.h"
#include "sier2View.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSier2View

IMPLEMENT_DYNCREATE(CSier2View, CView)

BEGIN_MESSAGE_MAP(CSier2View, CView)
	//{{AFX_MSG_MAP(CSier2View)
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_WM_ERASEBKGND()
	ON_COMMAND(IDM_POWER_0, OnPower0)
	ON_COMMAND(IDM_POWER_1, OnPower1)
	ON_COMMAND(IDM_POWER_2, OnPower2)
	ON_COMMAND(IDM_POWER_3, OnPower3)
	ON_COMMAND(IDM_POWER_4, OnPower4)
	ON_COMMAND(IDM_POWER_5, OnPower5)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSier2View construction/destruction


CSier2View::CSier2View(void)
{
}


CSier2View::~CSier2View(void)
{
}





BOOL CSier2View::PreCreateWindow(CREATESTRUCT& cs)
{

    /* OpenGL can render only into the client area of a window that has been
     initialized for OpenGL; it cannot render into child windows or siblings 
     of the window.  Therefore, we need to make sure that we clip the client 
     area for children and siblings.  This is done by adding the assignment
     to cs below.  -- openGL Quick Start, Dale Rogerson, Microsoft
   */
  cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
  return CView::PreCreateWindow(cs);
}


/*
 * set_dxf_write
 *
   
   The CSier2Doc object has a class variable that is a pointer
   to a pyr_display object.  When the SaveAs menu entry is selected
   the pyr_write() function is called to write the Sierpinski pyramid
   out in DXF format.  This function initialize the pointer to the
   pyr_display object with the address of "sier".

   Object purests will probably hold their noses, since by passing
   these "call back" pointers around, I am violating the sanctity of
   the object.  It certainly is true that if one plays tricks like
   this, you better make sure that the object is live at the time the
   pointer is referenced.

   I guess that all I can say is that Windows programming can be pretty
   gross stuff and if you think this code lack object purity, you should
   see the stuff Microsoft publishes as examples.

 */
void CSier2View::set_dxf_write(void)
{
   // Get a pointer to the applications document
   // object
    CSier2Doc* sier_doc = GetDocument();

    sier_doc->set_dxf_write_callback( &sier );
}  // set_dxf_write



/*
 * set_rotate
 *

   The CSier2App object (in sier2.cpp) contains the OnIdle class
   function.  This function calls the rotate code in this (CSier2View)
   class via a "call back" pointer.  This class initializes the
   call back pointer.
 */
void CSier2View::set_rotate(void)
{

  // set the idle loop mode
  // This is a hack call off to CSier2App
  CSier2App* pApp = (CSier2App*) AfxGetApp();

  // pass the CSier2View class to SetIdleEvent
  pApp->SetIdleEvent( this );
} // CSier2View::set_rotate



/*
 * stop_rotate
 *
   Set to call back pointer used by the OnIdle function (see set_rotate,
   above) to NULL, to stop rotation of the Sierpinski pyramid.
 */
void CSier2View::stop_rotate(void)
{

  // stop the idle loop mode
  // This is a hack call off to CSier2App
  CSier2App* pApp = (CSier2App*) AfxGetApp();

  // pass the CSier2View class to SetIdleEvent
  pApp->SetIdleEvent(NULL);
} // CSier2View::Rotate



/*
 * spinDisplay
 *
   Roate the Sierpinski object.  This function is called via the
   call back pointer initialized in set_rotate.

 */
void CSier2View::spinDisplay(void)
{
  sier.incr_spin_vars();
  Invalidate(FALSE) ;
  UpdateWindow() ;
} // CSier2View::spinDisplay





/* 
 * CSier2View drawing
 *

   Redraw the Sierpinski pyramid.

 */

void CSier2View::OnDraw(CDC* pDC)
{
  CSier2Doc* pDoc = GetDocument();
  ASSERT_VALID(pDoc);
  
  // Select the palette.
  CPalette* ppalOld = NULL;
  if (pal.get_pPal())
    {
      ppalOld = pDC->SelectPalette(pal.get_pPal(), 0);
      pDC->RealizePalette();
    }
  
  // Make the HGLRC current
  BOOL bResult = wglMakeCurrent(pDC->m_hDC, m_hrc);
  if (!bResult) {
    TRACE("wglMakeCurrent Failed %x\r\n", GetLastError() ) ;
  }

  // Draw pyramid
  sier.draw_object();

  //Swap Buffers
  SwapBuffers(pDC->m_hDC) ;

  // select old palette if we altered it
  if (ppalOld) pDC->SelectPalette(ppalOld, 0); 

  wglMakeCurrent(NULL, NULL) ;
} // CSier2View::OnDraw

/////////////////////////////////////////////////////////////////////////////
// CSier2View diagnostics -- build by the Microsoft AppWizard

#ifdef _DEBUG
void CSier2View::AssertValid() const
{
	CView::AssertValid();
}

void CSier2View::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CSier2Doc* CSier2View::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CSier2Doc)));
	return (CSier2Doc*)m_pDocument;
}
#endif //_DEBUG


/*
 * On Create
 *
   Set up the palette and the rendering context.

   This code is copied directly from "OpenGL I: Quick Start" by
   Dale Rogerson, Microsoft Developer Network Library.
   
 */

int CSier2View::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
  if (CView::OnCreate(lpCreateStruct) == -1)
    return -1; 
  
  // Create a device context.
  CClientDC dc(this) ;

  //
  // Fill in the Pixel Format Descriptor
  //
  PIXELFORMATDESCRIPTOR pfd ;
  // initialize the pixel format descriptor structure to zero
  memset(&pfd,0, sizeof(PIXELFORMATDESCRIPTOR)) ;
  
  pfd.nSize = sizeof(PIXELFORMATDESCRIPTOR);   
  pfd.nVersion = 1 ;                           // Version number
  pfd.dwFlags =  PFD_DOUBLEBUFFER |            // Use double buffer
                 PFD_SUPPORT_OPENGL |          // Use OpenGL
                 PFD_DRAW_TO_WINDOW ;          // Pixel format is for a window.
  pfd.iPixelType = PFD_TYPE_RGBA ;
  pfd.cColorBits = 24;                         // 8-bit color
  pfd.cDepthBits = 32 ; 		       // 32-bit depth buffer
  pfd.iLayerType = PFD_MAIN_PLANE ;            // Layer type

  /* The return value of ChoosePixelFormat is a one-based index into the 
     possible pixel formats.  The index is not unique and can change 
     depending on the current display mode. */
  int nPixelFormat = ChoosePixelFormat(dc.m_hDC, &pfd);
  if (nPixelFormat == 0)
    {
      TRACE("ChoosePixelFormat Failed %d\r\n",GetLastError()) ;
      return -1 ;
    }

  TRACE("Pixel Format %d\r\n", nPixelFormat) ;

  /* SetPixelFormat takes a handle to a device context (DC) as its first 
     parameter, and sets the window associated with the device context
     to the appropriate pixel format.
   */
  BOOL bResult = SetPixelFormat(dc.m_hDC, nPixelFormat, &pfd);
  if (!bResult)
    {
      TRACE("SetPixelFormat Failed %d\r\n",GetLastError()) ;
      return -1 ;
    }
  
  /*
     Create a rendering context.

     The rendering context could be thought of as a port through which
     all openGL commands must pass.  The rendering context you create
     has the same pixel format as the device context with which it is
     associated.  A rendering context is not the same as a device
     context: A device context information for GDI, while a rendering
     context contains information for openGL.  In many ways, however,
     a rendering context is to openGL what a device context is to GDI.

     wglCreateContext returns an HGLRC, which is a handle to the rendering 
     context.  This is stored in the class variable of CSier2View.
     -- OpenGL I: Quick Start by Dale Rogerson

   */

  m_hrc = wglCreateContext(dc.m_hDC);
  if (!m_hrc)
    {
      TRACE("wglCreateContext Failed %x\r\n", GetLastError()) ;
      return -1;
    }
  
  // Create the palette
  pal.CreateRGBPalette(dc.m_hDC) ;
  
  return 0;
} // CSier2View::OnCreate



//
//	OutputGlError
//
void CSier2View::OutputGlError(char* label) 
{
	GLenum errorno = glGetError() ;
	if (errorno != GL_NO_ERROR)
		TRACE("%s had error: #(%d) %s\r\n", label, errorno, 
		      gluErrorString(errorno)) ;
} /* OutputGlError */



/* 
 * OnSize

  When mapping a 3-D coordinate to the 2-D screen, OpenGL must know
  the size of the client area.  In the OnSize function, the
  transformations needed to map 3-D coordinates to the screen are set
  up.

 */
void CSier2View::OnSize(UINT nType, int w/*idth*/, int h/*eight*/) 
{
  CView::OnSize(nType, w, h);
  
  TRACE0("OnSize\r\n") ;
  CView::OnSize(nType, w, h);
  
  if ( (w <= 0) || (h <= 0) ) return ;
  
  sier.set_width( w );
  sier.set_height( h );

  //
  // Make the rendering context m_hrc current
  //
  CClientDC dc(this) ;
  BOOL bResult = wglMakeCurrent(dc.m_hDC, m_hrc);
  if (!bResult)
    {
      TRACE("wglMakeCurrent Failed %x\r\n", GetLastError() ) ;
      return ;
    }

  sier.set_GL_size();
  //
  // No rendering context will be current.
  //
  wglMakeCurrent(NULL, NULL);	
}  // OnSize



/*
 * OnEraseBkgnd
 *
   The OnEraseBkgnd function is overloaded to always return TRUE to
   stop the program from painting the screen white before the openGL
   screen is rendered.  This avoids a screen "flash" on rendering.

 */
BOOL CSier2View::OnEraseBkgnd(CDC* pDC) 
{
  // return CView::OnEraseBkgnd(pDC);
  return TRUE;
}



/*
 * CSier2View::OnInitialUpdate
 *
   + Create the display list
   + Set up the OnIdle function in the "MainFrame" object
     to call the Sierpinski rotate function.

 */
void CSier2View::OnInitialUpdate() 
{

  TRACE0("OnInitialUpdate\r\n");

  // Since display list creation requires a device context, this
  // can't be done until the OnCreate initialization is done.
  //
  // Attach the window dc to OpenGL.
  //
  CClientDC dc(this) ;
  BOOL bResult = wglMakeCurrent(dc.m_hDC, m_hrc);	 
  if (!bResult)
    {
      TRACE("wglMakeCurrent Failed %x\r\n", GetLastError() ) ;
    }

  sier.build_display_list();
  wglMakeCurrent(NULL, NULL) ;

  set_rotate();  // set up call back for OnIdle rotation
                 // of object.
  set_dxf_write(); // set up call back for DXF write

  CView::OnInitialUpdate();

}  // OnInitialUpdate



/*
 * make_new_pyramid
 *
   + Stop the rotation of the pyramid (which consumes a lot of CPU
     cycles.
   + Delete the old pyramid and create a new one of power "power".
   + Start rotation up again
   + Invalidate the window to force a redraw.
 */
void CSier2View::make_new_pyramid( unsigned int power )
{
  stop_rotate();
  //
  // Attach the window dc to OpenGL.
  //
  CClientDC dc(this) ;
  BOOL bResult = wglMakeCurrent(dc.m_hDC, m_hrc);	 
  if (!bResult) {
    TRACE("wglMakeCurrent Failed %x\r\n", GetLastError() ) ;
  }
  sier.new_pyramid( power );
  wglMakeCurrent(NULL, NULL) ;

  set_rotate();
  Invalidate(FALSE) ;
  UpdateWindow() ;
} // make_new_pyramid


/*
  Message handler functions for setting the "power" of the Sierpinski
  pyramid.

  */

void CSier2View::OnPower0() 
{
  make_new_pyramid( (unsigned int)0 );
}

void CSier2View::OnPower1() 
{
  make_new_pyramid( (unsigned int)1 );
}

void CSier2View::OnPower2() 
{
  make_new_pyramid( (unsigned int)2 );
}

void CSier2View::OnPower3() 
{
  make_new_pyramid( (unsigned int)3 );
}

void CSier2View::OnPower4() 
{
  make_new_pyramid( (unsigned int)4 );
}

void CSier2View::OnPower5() 
{
  make_new_pyramid( (unsigned int)5 );
}
